@@ -1,6 +1,7 @@
 from explorer.utils import passes_blacklist, write_csv, swap_params, execute_query, execute_and_fetch_query, extract_params, shared_dict_update
 from django.db import models, DatabaseError
 from django.core.urlresolvers import reverse
+from django.contrib.auth import get_user_model
 
 MSG_FAILED_BLACKLIST = "Query failed the SQL blacklist."
 
@@ -9,10 +10,9 @@ class Query(models.Model):
     title = models.CharField(max_length=255)
     sql = models.TextField()
     description = models.TextField(null=True, blank=True)
-    created_by = models.CharField(max_length=255, null=True, blank=True)
+    created_by_user = models.ForeignKey(get_user_model(), null=True, blank=True)
     created_at = models.DateTimeField(auto_now_add=True)
-
-    params = {}
+    last_run_date = models.DateTimeField(auto_now=True)
 
     class Meta:
         ordering = ['title']
@@ -21,17 +21,11 @@ class Query(models.Model):
     def __unicode__(self):
         return unicode(self.title)
 
-    def passes_blacklist(self):
-        return passes_blacklist(self.final_sql())
+    def passes_blacklist(self, params=None):
+        return passes_blacklist(self.final_sql(params=params))
 
-    def final_sql(self):
-        return swap_params(self.sql, self.params)
-
-    def csv_report(self):
-        headers, data, duration, error = self.headers_and_data()
-        if error:
-            return error
-        return write_csv(headers, data)
+    def final_sql(self, params=None):
+        return swap_params(self.sql, params)
 
     def error_messages(self):
         if not self.passes_blacklist():
@@ -42,19 +36,32 @@ class Query(models.Model):
         except DatabaseError, e:
             return str(e)
 
-    def headers_and_data(self):
-        if not self.passes_blacklist():
+    def headers_and_data(self, params=None):
+        if not self.passes_blacklist(params):
             return [], [], None, MSG_FAILED_BLACKLIST
         try:
-            return execute_and_fetch_query(self.final_sql())
-        except DatabaseError, e:
+            return execute_and_fetch_query(self.final_sql(params))
+        except (DatabaseError, Warning), e:
             return [], [], None, str(e)
 
-    def available_params(self):
+    def available_params(self, param_values=None):
         p = extract_params(self.sql)
-        if self.params:
-            shared_dict_update(p, self.params)
+        if param_values:
+            shared_dict_update(p, param_values)
         return p
 
     def get_absolute_url(self):
-        return reverse("query_detail", kwargs={'query_id': self.id})
\ No newline at end of file
+        return reverse("query_detail", kwargs={'query_id': self.id})
+
+    def log(self, user):
+        log_entry = QueryLog(sql=self.sql, query_id=self.id, run_by_user=user, is_playground=not bool(self.id))
+        log_entry.save()
+
+
+class QueryLog(models.Model):
+
+    sql = models.TextField()
+    query = models.ForeignKey(Query, null=True, blank=True, on_delete=models.SET_NULL)
+    is_playground = models.BooleanField(default=False)
+    run_by_user = models.ForeignKey(get_user_model(), null=True, blank=True)
+    run_at = models.DateTimeField(auto_now_add=True)
\ No newline at end of file
